@@ -5,4 +5,5 @@ from ._widget import PalmariWidget
 from .data_structure.experiment import Experiment
 from .data_structure.acquisition import Acquisition
 from .processing.tif_pipeline import TifPipeline
-from .processing.tif_pipeline_widget import TifPipelineWidget
+
+# from .processing.tif_pipeline_widget import TifPipelineWidget
